# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['webpack_loader',
 'webpack_loader.contrib',
 'webpack_loader.contrib.pages',
 'webpack_loader.contrib.pages.templatetags',
 'webpack_loader.templatetags']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'django-webpack5-loader',
    'version': '0.9.0',
    'description': 'Transparently use webpack with django',
    'long_description': '# django-webpack5-loader\n\n[![PyPI version](https://badge.fury.io/py/django-webpack5-loader.svg)](https://badge.fury.io/py/django-webpack5-loader)\n[![Build Status](https://circleci.com/gh/MrP01/django-webpack-loader/tree/master.svg?style=svg)](https://circleci.com/gh/owais/django-webpack-loader/tree/master)\n[![Coverage Status](https://coveralls.io/repos/MrP01/django-webpack-loader/badge.svg?branch=master&service=github)](https://coveralls.io/github/owais/django-webpack-loader?branch=master)\n![pyversions](https://img.shields.io/pypi/pyversions/django-webpack5-loader)\n![djversions](https://img.shields.io/pypi/djversions/django-webpack5-loader)\n\nUse webpack to generate your static bundles without django\'s staticfiles or opaque wrappers.\n\nDjango webpack loader consumes the output generated\nby [webpack-bundle-tracker](https://github.com/owais/webpack-bundle-tracker) and lets you use the generated bundles in\ndjango. **If you want to use entrypoints**, please use [webpack4-bundle-tracker](https://www.npmjs.com/package/webpack4-bundle-tracker).\n\nIf you want to leverage webpack critical css generation while maintaining\na Django-like structure in your project, have a look at [webpack-critical-pages](https://www.npmjs.com/package/webpack-critical-pages).\n\nA [changelog](CHANGELOG.md) is also available.\n\n## Purpose of this Fork\n\nThis package aims not only to support webpack 4 and 5, introduce additional loading of entrypoints in templates,\nbut also adds a framework for structuring django projects into pages using `webpack_loader.contrib.pages`.\nSee below.\n\nThe [initial webpack 4 port](https://github.com/alihazemfarouk/django-webpack-loader) was made possible thanks to\n\n- Svetoslav Dekov\n- Ali Farouk\n- Nikolaus Piccolotto\n\n## Install\n\n```bash\nnpm install --save-dev webpack-bundle-tracker\n\npip install django-webpack-loader\n```\n\n## Configuration\n\n### Configuring `webpack-bundle-tracker`\n\nBefore configuring `django-webpack-loader`, let\'s first configure what\'s necessary on `webpack-bundle-tracker` side. Update your Webpack configuration file (it\'s usually on `webpack.config.js` in the project root). Make sure your file looks like this (adapt to your needs):\n\n```javascript\nconst path = require("path");\nconst webpack = require("webpack");\nconst BundleTracker = require("webpack-bundle-tracker");\n\nmodule.exports = {\n  context: __dirname,\n  entry: "./assets/js/index",\n  output: {\n    path: path.resolve("./assets/webpack_bundles/"),\n    filename: "[name]-[hash].js",\n  },\n  plugins: [new BundleTracker({ filename: "./webpack-stats.json" })],\n};\n```\n\nThe configuration above expects the `index.js` (the app entrypoint file) to live inside the `/assets/js/` directory (this guide going forward will assume that all front-end related files are placed inside the `/assets/` directory, with the different kinds of files arranged within its subdirectories).\n\nThe generated compiled files will be placed inside the `/assets/webpack_bundles/` directory and the file with the information regarding the bundles and assets (`webpack-stats.json`) will be stored in the project root.\n\n### Compiling the front-end assets\n\nYou must generate the front-end bundle using `webpack-bundle-tracker` before using `django-webpack-loader`. You can compile the assets and generate the bundles by running:\n\n```bash\nnpx webpack --config webpack.config.js --watch\n```\n\nThis will also generate the stats file. You can also refer to how `django-react-boilerplate` configure the [package.json](https://github.com/vintasoftware/django-react-boilerplate/blob/master/package.json) scripts for different situations.\n\n> ⚠️ Hot reload is available through a specific config. Check [this section](#hot-reload).\n\n> ⚠️ This is the recommended usage for the development environment. For **usage in production**, please refer to [this section](#usage-in-production)\n\n### Configuring the settings file\n\nFirst of all, add `webpack_loader` to `INSTALLED_APPS`.\n\n```python\nINSTALLED_APPS = (\n  ...\n  \'webpack_loader\',\n  ...\n)\n```\n\nBelow is the recommended setup for the Django settings file when using `django-webpack-loader`.\n\n```python\nWEBPACK_LOADER = {\n  \'DEFAULT\': {\n    \'CACHE\': not DEBUG,\n    \'STATS_FILE\': os.path.join(BASE_DIR, \'webpack-stats.json\'),\n    \'POLL_INTERVAL\': 0.1,\n    \'IGNORE\': [r\'.+\\.hot-update.js\', r\'.+\\.map\'],\n  }\n}\n```\n\nFor that setup, we\'re using the `DEBUG` variable provided by Django. Since in a production environment (`DEBUG = False`) the assets files won\'t constantly change, we can safely cache the results (`CACHE=True`) and optimize our flow, as `django-webpack-loader` will read the stats file only once and store the assets files paths in memory. From that point onwards, it will use these stored paths as the source of truth. If `CACHE=False`, we\'ll always read the stats file to get the assets paths.\n\n> ⚠️ If `CACHE=True`, any changes made in the assets files will only be read when the web workers are restarted.\n\nDuring development, when the stats file changes a lot, we want to always poll for its updated version (in our case, we\'ll fetch it every 0.1s, as defined on `POLL_INTERVAL`).\n\n> ⚠️ In production (`DEBUG=False`), we\'ll only fetch the stats file once, so `POLL_INTERVAL` is ignored.\n\nWhile `CACHE` isn\'t directly related to `POLL_INTERVAL`, it\'s interesting to keep `CACHE` binded to the `DEBUG` logic value (in this case, the negation of the logic value) in order to only cache the assets in production, as we\'d not continuously poll the stats file in that environment.\n\nThe `STATS_FILE` parameter represents the output file produced by `webpack-bundle-tracker`. Since in the Webpack configuration file we\'ve named it `webpack-stats.json` and stored it on the project root, we must replicate that setting on the back-end side.\n\n`IGNORE` is a list of regular expressions. If a file generated by Webpack matches one of the expressions, the file will not be included in the template.\n\n### Extra settings\n\n- `TIMEOUT` is the number of seconds webpack_loader should wait for webpack to finish compiling before raising an exception. `0`, `None` or leaving the value out of settings disables timeouts\n\n- `INTEGRITY` is flag enabling [Subresource Integrity](https://developer.mozilla.org/en-US/docs/Web/Security/Subresource_Integrity) on rendered `<script>` and `<link>` tags. Integrity hash is get from stats file and configuration on side of `BundleTracker`, where configuration option `integrity: true` is required.\n\n- `LOADER_CLASS` is the fully qualified name of a python class as a string that holds the custom webpack loader. This is where behavior can be customized as to how the stats file is loaded. Examples include loading the stats file from a database, cache, external url, etc. For convenience, `webpack_loader.loader.WebpackLoader` can be extended. The `load_assets` method is likely where custom behavior will be added. This should return the stats file as an object.\n\nHere\'s a simple example of loading from an external url:\n\n```py\nimport requests\nfrom webpack_loader.loader import WebpackLoader\n\nclass ExternalWebpackLoader(WebpackLoader):\n  def load_assets(self):\n    url = self.config[\'STATS_URL\']\n    return requests.get(url).json()\n```\n\n## Rendering\n\nIn order to render the front-end code into the Django templates, we use the `render_bundle` template tag.\n\nIts behavior is to accept a string with the name of an entrypoint from the stats file (in our case, we\'re using `main`, which is [the default](https://webpack.js.org/concepts/entry-points/#single-entry-shorthand-syntax)) and it\'ll proceed to include all files under that entrypoint. You can read more about the entrypoints concept [here](https://webpack.js.org/concepts/entry-points/).\n\n> ⚠️ You can also check an example on how to use multiple `entry` values [here](https://github.com/django-webpack/django-webpack-loader/tree/master/examples/code-splitting).\n\nBelow is the basic usage for `render_bundle` within a template:\n\n#### render_bundle\n\n```HTML+Django\n{% load render_bundle from webpack_loader %}\n\n{% render_bundle \'main\' %}\n```\n\nThat will render the proper `<script>` and `<link>` tags needed in your template.\n\n## Running in development\n\nFor `django-webpack-loader` to work, you must run the webpack pipeline. Please refer to [this section](#compiling-the-front-end-assets).\n\nIn summary, you should do the following:\n\n```bash\n# in one shell\nnpx webpack --config webpack.config.js --watch\n\n# in another shell\npython manage.py runserver\n```\n\n> ⚠️ You can also check [this example](https://github.com/django-webpack/django-webpack-loader/tree/master/examples/simple) on how to run a project with `django-webpack-loader` and `webpack-bundle-track`.\n\n## Usage in production\n\nWe recommend that you keep your local bundles and the stats file outside the version control, having a production pipeline that will compile and collect the assets during the deployment phase.\n\nYou must add `STATICFILES_DIRS` to your settings file, pointing to the directory where the static files are located. This will let `collectstatic` know where it should look at:\n\n```python\nSTATICFILES_DIRS = (\n  os.path.join(BASE_DIR, \'assets\'),\n)\n```\n\nBelow are the commands that should be run to compile and collect the static files (please note this may change from platform to platform):\n\n```\nnpm run build\npython manage.py collectstatic --noinput\n```\n\nFirst we build the assets and, since we have `webpack-bundle-tracker` in our front-end building pipeline, the stats file will be populated. Then, we manually run collecstatic to collect the compiled assets.\n\n> ⚠️ Heroku is one platform that automatically runs collectstatic for you, so you need to set `DISABLE_COLLECTSTATIC=1` environment var. Instead, you must manually run collectstatic after running webpack. In Heroku, this is achieved with a `post_compile` hook. You can see an example on how to implement this flow on [django-react-boilerplate](https://github.com/vintasoftware/django-react-boilerplate/tree/master/bin).\n\nHowever, production usage for this package is **fairly flexible**. Other approaches may include keeping the production bundles in the version control and take that responsibility from the automatic pipeline. However, you must remember to always build the frontend and generate the bundle before pushing to remote.\n\n## Usage in tests\n\nThere are 2 approaches for when `render_bundle` shows up in tests, since we don\'t have `webpack-bundle-tracker` at that point to generate the stats file.\n\n1. The first approach is to have specific settings for them (which is how we approach on our [tests](https://github.com/django-webpack/django-webpack-loader/blob/master/tests/app/settings.py#L111-L125)), such as done [here](https://github.com/django-webpack/django-webpack-loader/issues/187#issuecomment-470055769). Please note that it\'s necessary to have a pre-made stats file for the tests (which in general can be empty, such as [here](https://github.com/django-webpack/django-webpack-loader/issues/187#issuecomment-464250721)).\n\n2. The second approach is to leverage [`LOADER_CLASS` overriding](#extra-settings) for the test settings and customize the `get_bundle` method to return the url of a stats file. Note that, using this approach, the stats file doesn\'t have to [exist](https://github.com/django-webpack/django-webpack-loader/issues/187#issuecomment-901449290).\n\n## Advanced Usage\n\n### Rendering by file extension\n\n`render_bundle` also takes a second argument which can be a file extension to match. This is useful when you want to\nrender different types for files in separately. For example, to render CSS in head and JS at bottom we can do something\nlike this,\n\n```HTML+Django\n{% load render_bundle from webpack_loader %}\n\n<html>\n  <head>\n    {% render_bundle \'main\' \'css\' %}\n  </head>\n  <body>\n    ....\n    {% render_bundle \'main\' \'js\' %}\n  </body>\n</head>\n```\n\n#### render_entrypoint (Available only when using Webpack v.4 or newer)\n\n```HTML+Django\n{% load render_entrypoint from webpack_loader %}\n\n{% render_entrypoint \'index\' %}\n```\n\n`render_entrypoint` will render all the proper `<script>` and `<link>` tags needed in your template for that endpoint.\nUsing this, you can make use of webpack 4 code splitting features. Example webpack config:\n\n```javascript\nmodule.exports = {\n  ...,\n  entry: {\n    index: "./myapp/static/src/pages/index.js",\n    contact_us: "./myapp/static/src/pages/contact_us.js",\n  },\n  ...,\n  plugins: [new BundleTracker({filename: "./webpack-stats.json"})]\n};\n\n```\n\nJust as `render_bundle`, `render_entrypoint` also takes a second argument which can be a file extension to match, and\ncan be used in a similar way,\n\n```HTML+Django\n{% load render_entrypoint from webpack_loader %}\n\n<html>\n  <head>\n    {% render_entrypoint \'main\' \'css\' %}\n  </head>\n  <body>\n    ....\n    {% render_entrypoint \'main\' \'js\' %}\n  </body>\n</head>\n```\n\n### Automatic page handling based on Entrypoints\n\nUsing `webpack_loader.contrib.pages` you can register entrypoints for corresponding pages in templates.\n\nAt the top of your individual page, do:\n\n```HTML+Jinja\n{% extends "layout.jinja" %}\n{% do register_entrypoint("myapp/dashboard") %}\n```\n\nIn the layout\'s (base template\'s) head, place the following:\n\n```HTML+Jinja\n<!DOCTYPE html>\n{% do register_entrypoint("main") %}\n<html lang="{{ LANGUAGE_CODE }}">\n<head>\n  ...\n  {{ render_css() }}\n</head>\n<body>\n  ...\n  {{ render_js() }}\n</body>\n```\n\nThis will load the registered entrypoints in order (`main`, then `myapp/dashboard`) and automatically inject\nthe webpack-generated css and js. It also supports critical css injection upon first request visits.\n\n```HTML+Jinja\n<html>\n  <head>\n    {% render_bundle \'main\' \'css\' is_preload=True %}\n    {% render_bundle \'main\' \'js\' is_preload=True %}\n\n    {% render_bundle \'main\' \'css\' %}\n\n  </head>\n\n  <body>\n    {% render_bundle \'main\' \'js\' %}\n  </body>\n</html>\n```\n\n### Accessing other webpack assets\n\n`webpack_static` template tag provides facilities to load static assets managed by webpack in Django templates. It is like Django\'s built in `static` tag but for webpack assets instead.\n\nIn the below example, `logo.png` can be any static asset shipped with any npm package.\n\n```HTML+Django\n{% load webpack_static from webpack_loader %}\n\n<!-- render full public path of logo.png -->\n<img src="{% webpack_static \'logo.png\' %}"/>\n```\n\nThe public path is based on `webpack.config.js` [output.publicPath](https://webpack.js.org/configuration/output/#output-publicpath).\n\nPlease note that this approach will use the original asset file, and not a post-processed one from the Webpack pipeline, in case that file had gone through such flow (i.e.: You\'ve imported an image on the React side and used it there, the file used within the React components will probably have a hash string on its name, etc. This processed file will be different than the one you\'ll grab with `webpack_static`).\n\n### Use `skip_common_chunks` on `render_bundle`\n\nYou can use the parameter `skip_common_chunks=True` to specify that you don\'t want an already generated chunk be included again in the same page. This should only happen if you use more than one entrypoint per Django template (multiple `render_bundle` calls). By using `skip_common_chunks=True`, you can get the same default behavior of the [HtmlWebpackPlugin](https://webpack.js.org/plugins/html-webpack-plugin/).\n\nIn order for this option to work, `django-webpack-loader` requires the `request` object to be in the context, to be able to keep track of the generated chunks.\n\nThe `request` object is passed by default via the `django.template.context_processors.request` middleware with using the Django built-in templating system, and also with using Jinja2.\n\nIf you don\'t have `request` in the context for some reason (e.g. using `Template.render` or `render_to_string` directly without passing the request), you\'ll get warnings on the console and the common chunks will remain duplicated.\n\n### Appending file extensions\n\nThe `suffix` option can be used to append a string at the end of the file URL. For instance, it can be used if your webpack configuration emits compressed `.gz` files.\nqwe\n\n```HTML+Django\n{% load render_bundle from webpack_loader %}\n<html>\n  <head>\n    <meta charset="UTF-8">\n    <title>Example</title>\n    {% render_bundle \'main\' \'css\' %}\n  </head>\n  <body>\n    {% render_bundle \'main\' \'js\' suffix=\'.gz\' %}\n  </body>\n</html>\n```\n\n### Multiple Webpack configurations\n\nVersion 1.0 and up of `django-webpack-loader` also supports multiple Webpack configurations. The following configuration defines 2 Webpack stats files in settings and uses the `config` argument in the template tags to influence which stats file to load the bundles from.\n\n```python\nWEBPACK_LOADER = {\n    \'DEFAULT\': {\n        \'BUNDLE_DIR_NAME\': \'bundles/\',\n        \'STATS_FILE\': os.path.join(BASE_DIR, \'webpack-stats.json\'),\n    },\n    \'DASHBOARD\': {\n        \'BUNDLE_DIR_NAME\': \'dashboard_bundles/\',\n        \'STATS_FILE\': os.path.join(BASE_DIR, \'webpack-stats-dashboard.json\'),\n    }\n}\n```\n\n```HTML+Django\n{% load render_bundle from webpack_loader %}\n\n<html>\n  <body>\n    ....\n    {% render_bundle \'main\' \'js\' \'DEFAULT\' %}\n    {% render_bundle \'main\' \'js\' \'DASHBOARD\' %}\n\n    <!-- or render all files from a bundle -->\n    {% render_bundle \'main\' config=\'DASHBOARD\' %}\n\n    <!-- the following tags do the same thing -->\n    {% render_bundle \'main\' \'css\' \'DASHBOARD\' %}\n    {% render_bundle \'main\' extension=\'css\' config=\'DASHBOARD\' %}\n    {% render_bundle \'main\' config=\'DASHBOARD\' extension=\'css\' %}\n\n    <!-- add some extra attributes to the tag -->\n    {% render_bundle \'main\' \'js\' \'DEFAULT\' attrs=\'async charset="UTF-8"\'%}\n  </body>\n</head>\n```\n\n### File URLs instead of html tags\n\nIf you need the URL to an asset without the HTML tags, the `get_files` template tag can be used. A common use case is specifying the URL to a custom css file for a Javascript plugin.\n\n`get_files` works exactly like `render_bundle` except it returns a list of matching files and lets you assign the list to a custom template variable.\n\nEach object in the returned list has 2 properties:\n\n1. `name`, which is the name of a chunk from the stats file;\n2. `url`, which can be:\n3. The `publicPath` if the asset has one;\n4. The `path` to the asset in the static files storage, if the asset doesn\'t have a `publicPath`.\n\nFor example:\n\n```HTML+Django\n{% load get_files from webpack_loader %}\n\n{% get_files \'editor\' \'css\' as editor_css_files %}\nCKEDITOR.config.contentsCss = \'{{ editor_css_files.0.url }}\';\n\n<!-- or list down name and url for every file -->\n<ul>\n{% for css_file in editor_css_files %}\n    <li>{{ css_file.name }} : {{ css_file.url }}</li>\n{% endfor %}\n</ul>\n```\n\n### Code splitting\n\nIn case you wish to use [code-splitting](https://webpack.js.org/guides/code-splitting/), follow the recipe below on the Javascript side.\n\nCreate your entrypoint file and add elements to the DOM, while leveraging the lazy imports.\n\n```js\n// src/principal.js\nfunction getComponent() {\n  return import(/* webpackChunkName: "lodash" */ "lodash")\n    .then(({ default: _ }) => {\n      const element = document.createElement("div");\n\n      element.innerHTML = _.join(["Hello", "webpack"], " ");\n\n      return element;\n    })\n    .catch((error) => "An error occurred while loading the component");\n}\n\ngetComponent().then((component) => {\n  document.body.appendChild(component);\n});\n```\n\nOn your configuration file, do the following:\n\n```js\n// webpack.config.js\nmodule.exports = {\n  context: __dirname,\n  entry: {\n    principal: "./src/principal",\n  },\n  output: {\n    path: path.resolve("./dist/"),\n    // publicPath should match your STATIC_URL config.\n    // This is required otherwise webpack will try to fetch\n    // our chunk generated by the dynamic import from "/" instead of "/dist/".\n    publicPath: "/dist/",\n    chunkFilename: "[name].bundle.js",\n    filename: "[name]-[hash].js",\n  },\n  plugins: [new BundleTracker({ filename: "./webpack-stats.json" })],\n};\n```\n\nIf you\'re using Webpack 5 instead of 4, do the following:\n\n- Change `filename: "[name]-[hash].js"` to `filename: "[name]-[fullhash].js"`;\n- Remove `/* webpackChunkName: "lodash" */`, which is not needed anymore.\n\nOn your template, render the bundle as usual:\n\n```HTML+Django\n<!-- index.html -->\n{% load render_bundle from webpack_loader %}\n\n<!DOCTYPE html>\n<html>\n  <head>\n    <meta charset="utf-8">\n    <title>My test page</title>\n  </head>\n  <body>\n    <p>This is my page</p>\n\n    {% render_bundle \'principal\' \'js\' %}\n  </body>\n</html>\n```\n\n### Hot reload\n\nIn case you wish to enable hot reload for your project using `django-webpack-loader` and `webpack-bundle-tracker`, please check out [this example](https://github.com/django-webpack/django-webpack-loader/tree/master/examples/hot-reload), in particular how [server.js](https://github.com/django-webpack/django-webpack-loader/blob/master/examples/hot-reload/server.js) and [webpack.config.js](https://github.com/django-webpack/django-webpack-loader/blob/master/examples/hot-reload/webpack.config.js) are configured.\n\n### Jinja2 Configuration\n\nIf you need to output your assets in a jinja template, we provide a Jinja2 extension that\'s compatible with [django-jinja](https://github.com/niwinz/django-jinja).\n\nTo install the extension, add it to the `TEMPLATES` configuration in the `["OPTIONS"]["extension"]` list.\n\n```python\nfrom django_jinja.builtins import DEFAULT_EXTENSIONS\nTEMPLATES = [\n  {\n    "BACKEND": "django_jinja.backend.Jinja2",\n    "OPTIONS": {\n      "extensions": DEFAULT_EXTENSIONS + [\n        "webpack_loader.contrib.jinja2ext.WebpackExtension",\n      ],\n    }\n  }\n]\n```\n\nThen in your base jinja template, do:\n\n```HTML\n{{ render_bundle(\'main\') }}\n```\n\n## Migrating from version < 1.0.0\n\nIn order to use `django-webpack-loader>=1.0.0`, you must ensure that `webpack-bundle-tracker@1.0.0` is being used on the JavaScript side. It\'s recommended that you always keep at least minor version parity across both packages, for full compatibility.\n\nThis is necessary because the formatting of `webpack-stats.json` that `webpack-bundle-tracker` outputs has changed starting at version `1.0.0-alpha.1`. Starting at `django-webpack-loader==1.0.0`, this is the only formatting accepted here, meaning that other versions of that package don\'t output compatible files anymore, thereby breaking compatibility with older `webpack-bundle-tracker` releases.\n\n## Commercial Support\n\n[![alt text](https://avatars2.githubusercontent.com/u/5529080?s=80&v=4 "Vinta Logo")](https://www.vinta.com.br/)\n\nThis project is maintained by [Vinta Software](https://www.vinta.com.br/) and is used in products of Vinta\'s clients. We are always looking for exciting work, so if you need any commercial support, feel free to get in touch: contact@vinta.com.br\n',
    'author': 'MrP01',
    'author_email': 'peter@waldert.at',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/MrP01/django-webpack-loader',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
